"""
Модуль с описанием класса Configuration
"""
# from .handlers import DBHandler
from .configuration import Configuration

__version__ = '2.3.2'
__name__ = 'py-configuration-Orinnass'

# TODO: изменить минимальный процент покрытия кода тестами
